<?php
// $Id: surveygizmo.api.php,v 1.1 2010/05/29 01:20:36 garbanzito Exp $

/**
 * @file
 * API functions for interacting with SurveyGizmo surveys.
 *
 */

// Define options for how a survey will be presented.
define('SURVEYGIZMO_AT_PATH', 1);
define('SURVEYGIZMO_ATTACHED_TO_NODE', 2);
// Define option to represent that the survey has been activated on the site.
define('SURVEYGIZOMO_ACTIVE', 1);
define('SURVEYGIZOMO_INACTIVE', 0);
// Define a status code used to filter requests from SurveyGizmo.
define('SURVEYGIZMO_STATUS_LAUNCHED', 'Launched');

// The key SurveyGizmo has defined for this module's access to the API.
define('SURVEYGIZMO_DEV_KEY', '8102fc186af77382b3e27591a');

// The name of the input filter used to process survey tokens.
define('SURVEYGIZMO_FILTER_NAME', 'SurveyGizmo embeddor');

// Pre-format a link with trademark suffix.
function surveygizmo_sponsor_link() {
  return l('SurveyGizmo', 'http://surveygizmo.com', array('attributes' => array('target' => '_blank'))) . "&trade;";
}


/**
 * Get the basic info about a survey stored on this site.
 *
 * @param $survey_id
 *   the id of the survey about which to get information.
 * @return
 *   array (sgid, active, type, path); returns default values and SURVEYGIZOMO_INACTIVE
 *   if no info stored for $survey_id.
 *
 */
function surveygizmo_get_survey($survey_id) {
  // Get stored info if any.
  $result = db_fetch_array(db_query("SELECT sgid, type, path FROM {surveygizmo} WHERE sgid = %d", $survey_id));
  if (!$result) {
    // Return default values.
    $result = array(
      'sgid' => $survey_id,
      'active' => SURVEYGIZOMO_INACTIVE,
      'type' => SURVEYGIZMO_AT_PATH, // 'At path',
      'path' => 'surveys/' . $survey_id,
    );
  }
  else {
    // If there's a record, the survey is active.
    $result['active'] = SURVEYGIZOMO_ACTIVE;
  }
  return $result;
}

/**
 * Get the basic info about all surveys stored on this site.
 *
 * @param $filter
 *   optional single field filter, associative array of field => value for the query
 *
 * @return array of arrays (sgid, active, type, path) of surveys which are active at the site
 * and which are valid at SurveyGizmo.
 *
 */
function surveygizmo_get_activated_surveys($filter = NULL) {
  $active_surveys = array();
  // Get a list of surveys for the keys on the site.
  $query = "SELECT sgid, type, path FROM {surveygizmo}";
  // @TODO: learn a better way to do the filter.
  if ($filter) {
    // Support only one filter for now. Assumes string or decimal values will always work.
    $key = array_keys($filter);
    $value = array_values($filter);
    $query = $query . " WHERE " . $key[0] . " = " . (gettype($value[0]) == 'string' ? "'%s'" : "%d");
    $param = $value[0];
  }
  $surveys = db_query($query, $param);
  while ($survey = db_fetch_array($surveys)) {
    $survey_obj = surveygizmo_get_survey_obj($survey['sgid']);
    if (is_object($survey_obj)) {
      $active_surveys[] = $survey;
    }
  }
  return $active_surveys;
}


/**
 * Return true if survey is activated on site.
 *
 * @param $survey_id the id of the survey to check.
 *
 */
function surveygizmo_is_active($survey_id) {
  $survey_info = surveygizmo_get_survey($survey_id);
  return ($survey_info['active'] == SURVEYGIZOMO_ACTIVE);
}

/**
 * Insert, update or delte survey data.
 *
 * @param $data key-value array of fields to store.
 * @param $status status flag indicating whether record should be deleted or retained/created.
 *
 */
function surveygizmo_set_survey($data, $status) {
  $s = surveygizmo_get_survey($data['sgid']);
  // If there is no existing record, so insert if requested.
  if (($s['active'] == SURVEYGIZOMO_INACTIVE) && ($status == SURVEYGIZOMO_ACTIVE)) {
    // Insert record.
    drupal_write_record('surveygizmo', $data);
  }
  // There is an existing record to either update or delete.
  else {
    // Deactivate survey by deleting existing record from storage.
    if ($status == SURVEYGIZOMO_INACTIVE) {
      $result = db_query('DELETE FROM {surveygizmo} WHERE sgid = %d', $data['sgid']);
    }
    else {
      // Update record.
      drupal_write_record('surveygizmo', $data, 'sgid');
    }
  }
}

/**
 * Return a survey simpleXMLobject, NULL for no such survey, or a string with a failure message.
 *
 * @param $survey_id the id of the survey to retreive.
 *
 */
function surveygizmo_get_survey_obj($survey_id) {
  $result = NULL;
  $s = surveygizmo_account_access();
  $list = $s->get_published_survey_list();
  if ((string)$surveys->status == 'failure') {
    $result = (string)$surveys->msg;
  }
  // Probably a timeout.
  elseif (!is_object($list)) {
    $result = t('There was an error getting this survey.');
  }
  else {
    $survey_xmlarray = $list->surveylist->survey;
    foreach ($survey_xmlarray as $survey) {
      if (($survey->id == $survey_id) && ($survey->status == SURVEYGIZMO_STATUS_LAUNCHED)) {
        $result = $survey;
        break;
      }
    }
  }
  return $result;
}


/**
 * Return a SurveyGizmo API access object primed with the site's SurveyGizmo API & developer keys.
 *
 */
function surveygizmo_account_access() {
  $s = new surveygizmo_api(variable_get('surveygizmo_api_key', ''), SURVEYGIZMO_DEV_KEY);
  return $s;
}

/**
 * Implement interface with surveygizmo.com.
 *
 * Assumes SurveyGizmo API version 1.5.
 *
 * See http://www.surveygizmo.com/survey-support/tutorials/online-survey-api-integration-cms-intranet/
 *
 */
class surveygizmo_api {
  // Requests are submitted to this URL.
  var $api_base_url = 'https://api.sgizmo.com/';
  var $user_key = '';
  var $dev_key = '';

  // Set up keys.
  // @param $user_key
  //    API key obtained from SurveyGizmo.com to access a user's surveys
  // @param $dev_key
  //    Fixed developer key to be used by all Drupal users of SurveyGizmo
  function __construct($user_key ='', $dev_key='') {
    $this->user_key = $user_key;
    $this->dev_key = $dev_key;
  }

  // Send a simple request to SurveyGizmo. No caching.
  // @param $request
  //    Request command string to pass to SurveyGizmo.
  // @param $params array
  //    Associative array of additional parameters to pass with the request.
  function _surveygizmo_request($request, $params=array()) {
    $base_params = array(
      'dk' => $this->dev_key,
      'uk'  => $this->user_key,
      'cmd' => $request,
    );
    $params = array_merge($base_params, $params);
    $query = $this->api_base_url . '?' . drupal_query_string_encode($params);
    $request_result = drupal_http_request($query);
    // Getting time-outs now and then.
    if ($request_result->error) {
      watchdog('SurveyGizmo', $request_result->error . ' HTTP Response code %code', array('%code'=>$request->code));
      drupal_set_message("Error connecting to SurveyGizmo");
      $result = NULL;
    }
    else {
      $result = simplexml_load_string($request_result->data);
    }
    return $result;
  }

  // Get a list of surveys from SurveyGizmo.
  // Note: This call respects the surveyid and the status parameters, however code elsewhere that
  // took advantage of that was factored out when the API wasn't functioning correctly.
  // Interaction with SurveyGizmo when the API key accesses many surveys could perform better if
  // use of these paramters were factored back in.
  //
  // @param $sid
  //    SurveyGizmo survey id
  function get_published_survey_list($sid = NULL) {
    $params = array('status' => SURVEYGIZMO_STATUS_LAUNCHED);
    if ($sid) {
      $params['surveyid'] = $sid;
    }
    return $this->_surveygizmo_request('getSurveyList', $params);
  }
}
